@@ -1,12 +1,12 @@
 from tqdm import tqdm
 import numpy as np
 import torch
-from typing import Callable, Optional, Union
-from alibi_detect.cd.base_online import BaseDriftOnline
+from typing import Any, Callable, Optional, Union
+from alibi_detect.cd.base_online import BaseMultiDriftOnline
 from alibi_detect.utils.pytorch import GaussianRBF, permed_lsdds, quantile
 
 
-class LSDDDriftOnlineTorch(BaseDriftOnline):
+class LSDDDriftOnlineTorch(BaseMultiDriftOnline):
     def __init__(
             self,
             x_ref: Union[np.ndarray, list],
@@ -32,7 +32,8 @@ class LSDDDriftOnlineTorch(BaseDriftOnline):
         x_ref
             Data used as reference distribution.
         ert
-            The expected run-time (ERT) in the absence of drift.
+            The expected run-time (ERT) in the absence of drift. For the multivariate detectors, the ERT is defined
+            as the expected run-time from t=0.
         window_size
             The size of the sliding test-window used to compute the test-statistic.
             Smaller windows focus on responding quickly to severe drift, larger windows focus on
@@ -90,15 +91,16 @@ class LSDDDriftOnlineTorch(BaseDriftOnline):
 
         # initialize kernel
         if sigma is None:
-            x_ref = torch.from_numpy(self.x_ref).to(self.device)
+            x_ref = torch.from_numpy(self.x_ref).to(self.device)  # type: ignore[assignment]
             self.kernel = GaussianRBF()
             _ = self.kernel(x_ref, x_ref, infer_sigma=True)
         else:
-            sigma = torch.from_numpy(sigma).to(self.device) if isinstance(sigma, np.ndarray) else None
-            self.kernel = GaussianRBF(sigma)
+            sigma = torch.from_numpy(sigma).to(self.device) if isinstance(sigma,  # type: ignore[assignment]
+                                                                          np.ndarray) else None
+            self.kernel = GaussianRBF(sigma)  # type: ignore[arg-type]
 
         if self.n_kernel_centers is None:
-            self.n_kernel_centers = 2*window_size
+            self.n_kernel_centers = 2 * window_size
 
         self._configure_kernel_centers()
         self._configure_thresholds()
@@ -108,7 +110,7 @@ class LSDDDriftOnlineTorch(BaseDriftOnline):
         x_ref = torch.from_numpy(self.x_ref).to(self.device)
         x_ref_means = x_ref.mean(0)
         x_ref_stds = x_ref.std(0)
-        self._normalize = lambda x: (x - x_ref_means)/(x_ref_stds + eps)
+        self._normalize = lambda x: (x - x_ref_means) / (x_ref_stds + eps)
         self.x_ref = self._normalize(x_ref).cpu().numpy()
 
     def _configure_kernel_centers(self):
@@ -117,7 +119,7 @@ class LSDDDriftOnlineTorch(BaseDriftOnline):
         self.c_inds, self.non_c_inds = perm[:self.n_kernel_centers], perm[self.n_kernel_centers:]
         self.kernel_centers = torch.from_numpy(self.x_ref[self.c_inds]).to(self.device)
         if np.unique(self.kernel_centers.cpu().numpy(), axis=0).shape[0] < self.n_kernel_centers:
-            perturbation = (torch.randn(self.kernel_centers.shape)*1e-6).to(self.device)
+            perturbation = (torch.randn(self.kernel_centers.shape) * 1e-6).to(self.device)
             self.kernel_centers = self.kernel_centers + perturbation
         self.x_ref_eff = torch.from_numpy(self.x_ref[self.non_c_inds]).to(self.device)  # the effective reference set
         self.k_xc = self.kernel(self.x_ref_eff, self.kernel_centers)
@@ -129,7 +131,7 @@ class LSDDDriftOnlineTorch(BaseDriftOnline):
         # test windows of size W (so 2W-1 test samples in total)
 
         w_size = self.window_size
-        etw_size = 2*w_size-1  # etw = extended test window
+        etw_size = 2 * w_size - 1  # etw = extended test window
         nkc_size = self.n - self.n_kernel_centers  # nkc = non-kernel-centers
         rw_size = nkc_size - etw_size  # rw = ref-window
 
@@ -139,7 +141,7 @@ class LSDDDriftOnlineTorch(BaseDriftOnline):
 
         # For stability in high dimensions we don't divide H by (pi*sigma^2)^(d/2)
         # Results in an alternative test-stat of LSDD*(pi*sigma^2)^(d/2). Same p-vals etc.
-        H = GaussianRBF(np.sqrt(2.)*self.kernel.sigma)(self.kernel_centers, self.kernel_centers)
+        H = GaussianRBF(np.sqrt(2.) * self.kernel.sigma)(self.kernel_centers, self.kernel_centers)
 
         # Compute lsdds for first test-window. We infer regularisation constant lambda here.
         y_inds_all_0 = [y_inds[:w_size] for y_inds in y_inds_all]
@@ -148,13 +150,13 @@ class LSDDDriftOnlineTorch(BaseDriftOnline):
         )
 
         # Can compute threshold for first window
-        thresholds = [quantile(lsdds_0, 1-self.fpr)]
+        thresholds = [quantile(lsdds_0, 1 - self.fpr)]
         # And now to iterate through the other W-1 overlapping windows
         p_bar = tqdm(range(1, w_size), "Computing thresholds") if self.verbose else range(1, w_size)
         for w in p_bar:
-            y_inds_all_w = [y_inds[w:(w+w_size)] for y_inds in y_inds_all]
+            y_inds_all_w = [y_inds[w:(w + w_size)] for y_inds in y_inds_all]
             lsdds_w, _ = permed_lsdds(self.k_xc, x_inds_all, y_inds_all_w, H, H_lam_inv=H_lam_inv)
-            thresholds.append(quantile(lsdds_w, 1-self.fpr))
+            thresholds.append(quantile(lsdds_w, 1 - self.fpr))
             x_inds_all = [x_inds_all[i] for i in range(len(x_inds_all)) if lsdds_w[i] < thresholds[-1]]
             y_inds_all = [y_inds_all[i] for i in range(len(y_inds_all)) if lsdds_w[i] < thresholds[-1]]
 
@@ -162,7 +164,7 @@ class LSDDDriftOnlineTorch(BaseDriftOnline):
         self.H_lam_inv = H_lam_inv
 
     def _configure_ref_subset(self):
-        etw_size = 2*self.window_size-1  # etw = extended test window
+        etw_size = 2 * self.window_size - 1  # etw = extended test window
         nkc_size = self.n - self.n_kernel_centers  # nkc = non-kernel-centers
         rw_size = nkc_size - etw_size  # rw = ref-window
         # Make split and ensure it doesn't cause an initial detection
@@ -178,26 +180,29 @@ class LSDDDriftOnlineTorch(BaseDriftOnline):
             h_init = self.c2s - self.k_xtc.mean(0)  # (Eqn 21)
             lsdd_init = h_init[None, :] @ self.H_lam_inv @ h_init[:, None]  # (Eqn 11)
 
-    def score(self, x_t: np.ndarray) -> Union[float, None]:
+    def _update_state(self, x_t: torch.Tensor):  # type: ignore[override]
+        self.t += 1
+        k_xtc = self.kernel(x_t, self.kernel_centers)
+        self.test_window = torch.cat([self.test_window[(1 - self.window_size):], x_t], 0)
+        self.k_xtc = torch.cat([self.k_xtc[(1 - self.window_size):], k_xtc], 0)
+
+    def score(self, x_t: Union[np.ndarray, Any]) -> float:
         """
         Compute the test-statistic (LSDD) between the reference window and test window.
-        If the test-window is not yet full then a test-statistic of None is returned.
 
         Parameters
         ----------
         x_t
-            A single instance.
+            A single instance to be added to the test-window.
 
         Returns
         -------
         LSDD estimate between reference window and test window.
         """
-        x_t = torch.from_numpy(x_t[None, :]).to(self.device)
+        x_t = super()._preprocess_xt(x_t)
+        x_t = torch.from_numpy(x_t).to(self.device)
         x_t = self._normalize(x_t)
-        k_xtc = self.kernel(x_t, self.kernel_centers)
-
-        self.test_window = torch.cat([self.test_window[(1-self.window_size):], x_t], 0)
-        self.k_xtc = torch.cat([self.k_xtc[(1-self.window_size):], k_xtc], 0)
+        self._update_state(x_t)
         h = self.c2s - self.k_xtc.mean(0)  # (Eqn 21)
         lsdd = h[None, :] @ self.H_lam_inv @ h[:, None]  # (Eqn 11)
         return float(lsdd.detach().cpu())
