@@ -9,3 +9,9 @@ try:
     has_pytorch = True
 except ImportError:
     has_pytorch = False
+
+try:
+    import sklearn  # noqa
+    has_sklearn = True
+except ImportError:
+    has_sklearn = False
