@@ -58,6 +58,7 @@ extensions = [
     "sphinxcontrib.bibtex",
     "nbsphinx",
     "myst_parser",
+    "sphinx_design",
 ]
 
 # -- nbsphinx settings -------------------------------------------------------
@@ -255,7 +256,10 @@ epub_exclude_files = ["search.html"]
 # -- Options for intersphinx extension ---------------------------------------
 
 # Example configuration for intersphinx: refer to the Python standard library.
-intersphinx_mapping = {"https://docs.python.org/": None}
+intersphinx_mapping = {
+        'python': ('https://docs.python.org/', None),
+        'sklearn': ('https://scikit-learn.org/stable/', None),
+        }
 
 # -- Options for todo extension ----------------------------------------------
 
